@@ -38,7 +38,16 @@ WPS8Parser public
 WPS8Parser::WPS8Parser(WPXInputStream *input, WPSHeader * header) :
 	WPSParser(input, header),
 	offset_eot(0),
-	oldTextAttributeBits(0)
+	oldTextAttributeBits(0),
+	headerIndexTable(),
+	CHFODs(),	
+	PAFODs(),	
+	fonts(),
+	streams(),
+	footnotes(),
+	endnotes(),
+	fn_iter(),
+	en_iter()
 {
 }
 
@@ -112,7 +121,7 @@ void WPS8Parser::readFontsTable(WPXInputStream * input)
 	if (fonts.size() != n_fonts)
 	{
 		WPS_DEBUG_MSG(("Works: warning: expected %i fonts but only found %i\n",
-			n_fonts, fonts.size()));
+			n_fonts, int(fonts.size())));
 	}
 }
 
@@ -157,32 +166,32 @@ void WPS8Parser::readStreams(WPXInputStream * input)
 	offset = readU32(input);
 	if (offset) { WPS_DEBUG_MSG(("Offset table is not 0-terminated!\n")); }
 
-	for (unsigned i=0; i < n_streams; i++) {
+	for (unsigned j=0; j < n_streams; j++) {
 		uint16_t len;
-		uint32_t unknown, type = 0;
+		uint32_t type = 0;
 
 		len = readU16(input);
 		if (len > 10) {
-			WPS_DEBUG_MSG(("Rogue strm[%d] def len (%d)\n",i,len));
+			WPS_DEBUG_MSG(("Rogue strm[%d] def len (%d)\n",j,len));
 			input->seek(len-2,WPX_SEEK_CUR);
 		}
 
 		if (len > 4) {
-			unknown = readU32(input); // assume == 0x22000000
+			readU32(input); // assume == 0x22000000
 			type = readU32(input);
 		} else input->seek(len-2,WPX_SEEK_CUR);
 
-		streams[i].type = type;
+		streams[j].type = type;
 	}
 
 #ifdef DEBUG
 	int bodypos = -1;
-	for (int i=0; i < n_streams; i++) {
-		int z = streams[i].type;
-		if (z == WPS_STREAM_DUMMY) WPS_DEBUG_MSG(("Default strm[%d] type\n",i));
+	for (unsigned k=0; k < n_streams; k++) {
+		int z = streams[k].type;
+		if (z == WPS_STREAM_DUMMY) WPS_DEBUG_MSG(("Default strm[%d] type\n",k));
 		if (z == WPS_STREAM_BODY) {
-			if (bodypos < 0) bodypos = i;
-			else WPS_DEBUG_MSG(("Duplicating body (strm[%d])\n",i));
+			if (bodypos < 0) bodypos = k;
+			else WPS_DEBUG_MSG(("Duplicating body (strm[%d])\n",k));
 		}
 	}
 	if (bodypos < 0) WPS_DEBUG_MSG(("Doc body not found!\n"));
@@ -582,15 +591,29 @@ void WPS8Parser::readTextRange(WPXInputStream * input, WPS8ContentListener *list
 							switch (spec) {
 								case 3:
 									if (stream != WPS_STREAM_BODY) break;
-									listener->openFootnote();
+								  /* REMOVED in this version
+								     because footnote/endnote tends to make
+								     the resulting file inconsistent */
+									//listener->openFootnote();
+									listener->insertCharacter('-');
+									listener->insertCharacter('-');
 									readNote(input,listener,false);
-									listener->closeFootnote();
+									listener->insertCharacter('-');
+									listener->insertCharacter('-');
+									//listener->closeEndnote();
 									break;
 								case 4:
 									if (stream != WPS_STREAM_BODY) break;
-									listener->openEndnote();
+								  /* REMOVED in this version
+								     because footnote/endnote tends to make
+								     the resulting file inconsistent */
+									//listener->openEndnote();
+									listener->insertCharacter('-');
+									listener->insertCharacter('-');
 									readNote(input,listener,true);
-									listener->closeEndnote();
+									listener->insertCharacter('-');
+									listener->insertCharacter('-');
+									//listener->closeEndnote();
 									break;
 								case 5:
 									listener->insertField();
@@ -609,6 +632,12 @@ void WPS8Parser::readTextRange(WPXInputStream * input, WPS8ContentListener *list
 						// ! fallback to default
 
 					default:
+						if (readVal < 28 && readVal != 9)
+						{
+							// do not add unprintable control which can create invalid odt file
+					      		WPS_DEBUG_MSG(("WPS8Parser::readTextRange(find unprintable character: ignored)\n"));
+							break;
+						}
 						// fixme: convert UTF-16LE to UTF-8
 						input->seek(-2, WPX_SEEK_CUR);
 						this->appendUTF16LE(input, listener);
@@ -661,8 +690,16 @@ void WPS8Parser::readNote(WPXInputStream * input, WPS8ContentListener *listener,
 	WPS_DEBUG_MSG(("Reading footnote [%d;%d)\n",note.contents.start,note.contents.limit));
 
 	uint32_t pos = input->tell();
+	uint32_t endPos = stream.span.start+note.contents.limit;
+	// try to remove the end of lines which can appear after the footnote
+	while (endPos-1 > stream.span.start+note.contents.start) {
+	  input->seek(0x200+2*(endPos-1),WPX_SEEK_SET);
+	  uint16_t readVal =readU16(input); 
+	  if (readVal != 0xd) break;
+	  endPos -= 1;
+	}
 	readTextRange(input,listener,stream.span.start+note.contents.start,
-		stream.span.start+note.contents.limit,streamkey);
+		endPos,streamkey);
 	input->seek(pos,WPX_SEEK_SET);
 }
 
@@ -683,7 +720,7 @@ bool WPS8Parser::readFODPage(WPXInputStream * input, std::vector<FOD> * FODs, ui
 	uint16_t cfod = readU16(input); /* number of FODs on this page */
 
 	//fixme: what is the largest possible cfod?
-	if (cfod > 0x50)
+	if (cfod > 0x54)
 	{
 		WPS_DEBUG_MSG(("Works8: error: cfod = %i (0x%X)\n", cfod, cfod));
 		throw ParseException();
@@ -727,7 +764,7 @@ bool WPS8Parser::readFODPage(WPXInputStream * input, std::vector<FOD> * FODs, ui
 		if ((*FODs_iter).fcLim == offset_eot)
 			break;
 
-                (*FODs_iter).bfprop = readU16(input);
+		(*FODs_iter).bfprop = readU16(input);
 
 		/* check size of bfprop  */
 		if (((*FODs_iter).bfprop < (8 + (6*cfod)) && (*FODs_iter).bfprop > 0) ||
@@ -764,8 +801,8 @@ bool WPS8Parser::readFODPage(WPXInputStream * input, std::vector<FOD> * FODs, ui
 			WPS_DEBUG_MSG(("Works: error: 0 == cch at file offset 0x%lx", (input->tell())-1));
 			throw ParseException();
 		}
-		// fixme: what is largest cch?
-		if ((*FODs_iter).fprop.cch > 128)
+		// check that the property remains in the FOD zone
+		if ((*FODs_iter).bfprop_abs+(*FODs_iter).fprop.cch > page_offset+page_size)
 		{
 			WPS_DEBUG_MSG(("Works: error: cch = %i, too large ", (*FODs_iter).fprop.cch));
 			throw ParseException();
@@ -801,9 +838,12 @@ void WPS8Parser::parseHeaderIndexEntry(WPXInputStream * input)
 	if (0x18 != cch)
 	{
 		WPS_DEBUG_MSG(("Works8: error: parseHeaderIndexEntry cch = %i (0x%X)\n", cch, cch));
-#if 0
-		throw ParseException();
-#endif
+               /* Osnola: normally, this size must be >= 0x18
+                  In my code, I throw an exception when this size is less than 10
+                  to try to continue the parsing ( but I do not accept this entry)
+                */
+               if (cch < 10)
+                 throw ParseException();
 	}
 
 	std::string name;
@@ -849,8 +889,8 @@ void WPS8Parser::parseHeaderIndexEntry(WPXInputStream * input)
 		name.c_str(), hie.offset, hie.length));
 
 	headerIndexTable.insert(std::multimap<std::string, HeaderIndexEntries>::value_type(name, hie));
-
-	input->seek(input->tell() + 0x18 - cch, WPX_SEEK_SET);
+       // OSNOLA: cch is the length of the entry, so we must advance by cch-0x18
+       input->seek(input->tell() + cch - 0x18, WPX_SEEK_SET);
 }
 
 /**
@@ -1032,7 +1072,7 @@ void WPS8Parser::parse(WPXInputStream *input, WPS8ContentListener *listener)
 void WPS8Parser::propertyChangeTextAttribute(const uint32_t newTextAttributeBits, const uint8_t attribute, const uint32_t bit, WPS8ContentListener *listener)
 {
 	if ((oldTextAttributeBits ^ newTextAttributeBits) & bit)
-		listener->attributeChange(newTextAttributeBits & bit, attribute);
+		listener->attributeChange(newTextAttributeBits & bit ? true : false, attribute);
 }
 
 /**
@@ -1138,7 +1178,9 @@ void WPS8Parser::propertyChange(std::string rgchProp, WPS8ContentListener *liste
 					break;
 				default:
 					WPS_DEBUG_MSG(("Works8: error: unknown 0x0A format code 0x%04X\n", rgchProp[x]));
- 					throw ParseException();
+                                       // OSNOLA: ok to continue, this is a bool field
+                                       // ( without any data )
+                                       //throw ParseException();
 			}
 			continue;
 		}
